@@ -90,7 +90,8 @@ def read_onnx(path):
             cur[0] = [cur[0][0], kname, bname]
             layers.append([i.name, 'batchnorm', {}])
         elif i.op_type == 'Conv':
-            attr, w = i.attribute, values[i.input[1]][1]
+            # attr, w = i.attribute, values[i.input[1]][1]
+            attr = i.attribute
             g = node(attr, 'group', 'i') or 1
             d = node(attr, 'dilations', 'ints')
             p = node(attr, 'pads', 'ints')
@@ -98,7 +99,7 @@ def read_onnx(path):
             layers.append([i.name, 'conv', {
                 'group':g, 'strides':s, 'dilations':d, 'pads':p}])
         elif i.op_type == 'ConvTranspose':
-            attr, w = i.attribute, values[i.input[1]][1]
+            attr = i.attribute
             para = {}
             g = node(attr, 'group', 'i', para)
             d = node(attr, 'dilations', 'ints', para)
@@ -128,26 +129,43 @@ def read_onnx(path):
             layers.append([i.name, 'flatten', {}])
         elif i.op_type == 'Unsqueeze':
             axis = node(i.attribute, 'axes', 'ints')
-            layers.append([i.name, 'unsqueeze', {} if axis is None else {'axis':axis}])
+            layers.append([i.name, 'unsqueeze', {} if axis is None else {'axes':axis}])
+        elif i.op_type == 'Squeeze':
+            axis = node(i.attribute, 'axes', 'ints')
+            layers.append([i.name, 'squeeze', {} if axis is None else {'axes':axis}])
         elif i.op_type == 'Relu':
             layers.append([i.name, 'relu', {}])
         elif i.op_type == 'LeakyRelu':
             alpha = i.attribute[0].f
             layers.append([i.name, 'leakyrelu', {'alpha':alpha}])
+        elif i.op_type == 'HardSigmoid':
+            para = {}
+            node(i.attribute, 'alpha', 'f', para)
+            node(i.attribute, 'beta', 'f', para)
+            layers.append([i.name, 'hardsigmoid', para])
         elif i.op_type == 'Add':
             layers.append([i.name, 'add', {}])
         elif i.op_type == 'Sub':
             layers.append([i.name, 'sub', {}])
         elif i.op_type == 'Div':
             layers.append([i.name, 'div', {}])
+        elif i.op_type == 'Tile':
+            layers.append([i.name, 'tile', {}])
+        elif i.op_type == 'MatMul':
+            layers.append([i.name, 'matmul', {}])
         elif i.op_type == 'Constant':
+            _, _, name = flows.pop(-1)
             dim = i.attribute[0].t.dims
-
-            buf = i.attribute[0].t.raw_data
             tp = types[i.attribute[0].t.data_type]
-            # if len(buf)==0: continue
-            v = numpy.frombuffer(buf, tp).reshape(dim).tolist()
-            layers.append([i.name, 'const', {'value':v, 'dtype':tp}])
+
+            v = onnx.numpy_helper.to_array(i.attribute[0].t)
+            values[name] = len(weights), v.shape
+            inits.append([name, v.shape, str(v.dtype)])
+            if v.ndim==0: v = np.array([v])
+            weights.append(v)
+            #layers.append([i.name, 'const', {'value':v, 'dtype':tp}])
+        elif i.op_type == 'Identity':
+            layers.append([i.name, 'identity', {}])
         elif i.op_type == 'Pow':
             layers.append([i.name, 'pow', {}])
         elif i.op_type == 'ReduceSum':
@@ -168,12 +186,14 @@ def read_onnx(path):
         elif i.op_type == 'Sigmoid':
             layers.append([i.name, 'sigmoid', {}])
         elif i.op_type == 'AveragePool':
-            return 'lost', i
-            print('AveragePool IO, need review')
-            for attr in i.attribute:
-                if 'stride' in attr.name: s = attr.ints[0]
-                if 'kernel' in attr.name: w = attr.ints[0]
-            layers.append([i.name, 'avgpool', [w, s]])
+            w = node(i.attribute, 'kernel_shape', 'ints')
+            m = node(i.attribute, 'pads', 'ints')
+            s = node(i.attribute, 'strides', 'ints')
+            layers.append([i.name, 'averagepool', {'w':w, 'pads':m, 'strides':s}])
+        elif i.op_type == 'LSTM':
+            para = {'hidden_size': i.attribute[0].i}
+            node(i.attribute, 'direction', 's', para)
+            layers.append([i.name, 'lstm', para])
         elif i.op_type == 'Shape':
             layers.append([i.name, 'shape', {}])
         elif i.op_type == 'Gather':
@@ -186,6 +206,8 @@ def read_onnx(path):
             layers.append([i.name, 'transpose', {'axis':node(i.attribute, 'perm', 'ints')}])
         elif i.op_type == 'LogSoftmax':
             layers.append([i.name, 'logsoftmax', {'axis':i.attribute[0].i}])
+        elif i.op_type == 'Softmax':
+            layers.append([i.name, 'softmax', {'axis':i.attribute[0].i}])
         elif i.op_type == 'ConstantOfShape': 
             dim = i.attribute[0].t.dims
             buf = i.attribute[0].t.raw_data
