@@ -1,4 +1,4 @@
-from .util import maxpool, upsample, avgpool, np
+from .util import maxpool, upsample, avgpool, np, lstm
 from .util import conv_for, conv_stride, conv_dnn
 ep, dnn = None, None # numexpr is help for numpy backend
 import numpy
@@ -17,6 +17,8 @@ def Dense(x, K, B, shp=None):
     y += B.reshape((1, -1))
     return y
 
+def MatMul(x, y): return x @ y
+
 def Conv2d(x, K, B=None, group=1, strides=(1,1), dilations=(1,1), pads=(0,0,0,0)):
     if np is numpy: out = conv_for(x, K, group, pads, strides, dilations)
     elif not dnn is None: return conv_dnn(x, K, B, group, pads, strides, dilations)
@@ -31,6 +33,14 @@ def ConvTranspose2d(x, K, B=None, strides=[2,2], dilations=[1,1], pads=[0,0,0,0]
     buf[:,:,low_h:buf.shape[2]-high_h:s1,low_w:buf.shape[3]-high_w:s2] = x
     return Conv2d(buf, K.transpose(1,0,2,3)[:,:,::-1,::-1], B, strides=[1,1], dilations=dilations, group=group)
 
+def LSTM(X, W, R, B=0, sequence_lens=0, initial_h=0, initial_c=0, hidden_size=None, direction='forward'):
+    dirs = {'forward':[1], 'reverse':[-1], 'bidirectional':[1,-1]}
+    (L, N, input_dim), hidden_size = X.shape, R.shape[-1]
+    Y = np.zeros((L, len(dirs[direction]), N, hidden_size), dtype=np.float32)
+    for i, d in enumerate(dirs[direction]):
+        _, H, C = lstm(X, Y[:,i], W[i], R[i], B[i], initial_h[i], initial_c[i], d)
+    return Y, H, C
+
 def ReLU(x): 
     if ep: return ep.evaluate('x * (x > 0)')
     return np.multiply(x, x>0, out=x)
@@ -40,6 +50,10 @@ def LeakyReLU(x, alpha=0.2):
     if ep: return ep.evaluate('x*((x>0)*b+a)')
     y = (x>0) * b; y += a; y *= x; return y
 
+def Identity(x): return x
+
+def Tile(x, repeat): return np.tile(x, repeat.tolist())
+
 def Flatten(x): return x.reshape((x.shape[0], -1))
 
 def Sigmoid(x):
@@ -47,14 +61,20 @@ def Sigmoid(x):
     x = -x; np.exp(x, out=x); x += 1
     return np.divide(1, x, out=x)
 
+def HardSigmoid(x, alpha=0.2, beta=0.5):
+    x = x * alpha; x += beta
+    x = np.minimum(x, 1, out=x)
+    return np.maximum(x, 0, out=x)
+
 def Softmax(x, axis=-1):
-    eX = np.exp((x.T - np.max(x, axis=self.axis)).T)
-    return (eX.T / eX.sum(axis=self.axis)).T
+    x = np.exp(x)
+    x /= x.sum(axis=axis, keepdims=1)
+    return x
 
 def Maxpool(x, w=(2,2), pads=(0,0,0,0), strides=(2,2)):
     return maxpool(x, w, pads, strides)
 
-def Avgpool(x, w=(2,2), pads=(0,0,0,0), strides=(2,2)):
+def AveragePool(x, w=(2,2), pads=(0,0,0,0), strides=(2,2)):
     return avgpool(x, w, pads, strides)
 
 def GlobalAveragePool(x):
@@ -83,7 +103,9 @@ def Sub(x1, x2):
 
 def Pow(x, p): return np.power(x, p)
     
-def Div(x1, x2): return x1 / x2
+def Div(x1, x2): 
+    if ep: return ep.evaluate('x1 / x2')
+    return x1 / x2
 
 def ReduceSum(x, axis=-1, keepdims=False):
     return x.sum(axis=tuple(axis), keepdims=keepdims)
@@ -95,10 +117,13 @@ def BatchNorm(x, K, B):
     if ep: return ep.evaluate('x * K + B')
     x = x * K; x += B; return x
 
-def Unsqueeze(x, axis=None): 
-    axis = np.array(axis).tolist()
+def Unsqueeze(x, axes=None): 
+    axis = np.array(axes).tolist()
     return np.expand_dims(x, tuple(axis))
 
+def Squeeze(x, axes=[0]):
+    return np.squeeze(x, axis=axes[0])
+
 def Mul(x1, x2): 
     if ep: return ep.evaluate('x1 * x2')
     return x1 * x2
@@ -118,9 +143,10 @@ def Shape(x): return np.array(x.shape)
 def Gather(x, idx, axis=0): return np.take(x, idx, axis=axis)
 
 def Reshape(x, shp): 
+    shp = shp.tolist()
     for i in range(len(shp)): 
         shp[i] = shp[i] or x.shape[i]
-    return x.reshape(shp.tolist())
+    return x.reshape(shp)
 
 def Transpose(x, axis): return x.transpose(axis)
 
@@ -190,18 +216,21 @@ def Pad(x, pads, constant_value=0, mode='constant'):
     return np.pad(x, pads, **para)
 
 def Clip(x, min=0, max=1): 
-    return np.clip(x, min, max, out=x)
+    x = np.minimum(x, max, out=x)
+    return np.maximum(x, min, out=x)
 
 def Return(*x): return x
 
 layer_map = {'dense': Dense, 'conv': Conv2d, 'relu': ReLU, 
              'leakyrelu': LeakyReLU, 'batchnorm': BatchNorm,
              'flatten': Flatten, 'sigmoid': Sigmoid, 'softmax': Softmax, 
-             'maxpool': Maxpool, 'avgpool': Avgpool, 'const': Const,
+             'hardsigmoid': HardSigmoid, 'squeeze': Squeeze,
+             'maxpool': Maxpool, 'averagepool': AveragePool, 'const': Const,
              'upsample': UpSample, 'concat': Concatenate, 'add': Add, 
              'resize': Resize, 'pad': Pad, 'convtranspose':ConvTranspose2d,
              'sub': Sub, 'reducemean': ReduceMean, 'exp': Exp, 'log': Log,
-             'mul': Mul, 'gap': GlobalAveragePool, 'pow':Pow,
+             'mul': Mul, 'gap': GlobalAveragePool, 'pow':Pow, 'matmul': MatMul,
+             'identity' : Identity, 'tile': Tile, 'lstm':LSTM,
              'reducesum':ReduceSum, 'div':Div, 'unsqueeze':Unsqueeze, 
              'shape': Shape, 'gather':Gather, 'reshape':Reshape,
              'split':Split, 'tanh':Tanh, 'constantofshape':ConstantofShape,
