@@ -40,7 +40,7 @@ def conv_for(img, core, group=1, pads=(1, 1, 1, 1), strides=(1, 1), dilation=(1,
     # ============================================
     col_core = core.reshape((group, core.shape[0]//group, -1))
     col_img = col_img.reshape(group, cimg_w//group, -1)
-    rst = [i.dot(j) for i, j in zip(col_core, col_img)]
+    rst = [i @ j for i, j in zip(col_core, col_img)]
     rst = rst[0] if group==1 else np.concatenate(rst)
     return rst.reshape((n, ni, nh, nw)).transpose(1, 0, 2, 3)
 
@@ -101,6 +101,25 @@ def avgpool(i, c=(2, 2), mar=(0,0,0,0), s=(2, 2)):
     rst /= c[0] * c[1]
     return rst
 
+def lstm(X, Y, w, r, b, ht, ct, dir=1):
+    def tanh(x): return np.tanh(x, out=x)
+
+    def sigmoid(x): 
+        x *= -1; np.exp(x, out=x); x += 1
+        return np.divide(1, x, out=x)
+
+    for t in list(range(X.shape[0]))[::dir]:
+        gates = np.dot(X[t:t+1], w.T)
+        gates += np.dot(ht, r.T)
+        gates += b[:b.shape[0]//2]
+        gates += b[b.shape[0]//2:]
+        i, o, f, c = np.split(gates, 4, -1)
+        sigmoid(i), sigmoid(f), tanh(c)
+        f *= ct; i *= c; f += i; C = f
+        o = sigmoid(o); o *= np.tanh(C)
+        Y[t], ht, ct = o[0], o[0], C
+    return Y, ht, ct
+
 def make_upmat(k):
     ys = np.linspace(0.5/k[0], 1-0.5/k[0], k[0]*1, dtype=np.float32)
     xs = np.linspace(0.5/k[1], 1-0.5/k[1], k[1]*1, dtype=np.float32)
@@ -211,7 +230,7 @@ def grid_slice(H, W, h, w, mar):
     return list(itertools.product(a, b))
 
 def resize(img, size, backend=None):
-    nn = backend or np
+    nn = np if isinstance(img, np.ndarray) else numpy
     d, (h, w) = img.ndim, img.shape[:2]
     kh, kw = size[0]/h, size[1]/w
     slicer = -0.5+0.5/kh, h-0.5-0.5/kh, size[0]
@@ -229,7 +248,7 @@ def resize(img, size, backend=None):
     return buf[ra,:]*(1-rs) + buf[rb,:]*rs
 
 def mapcoord(img, rs, cs, keeptp=True, backend=None):
-    nn = backend or np
+    nn = np if isinstance(img, np.ndarray) else numpy
     d, (h, w) = img.ndim, img.shape[:2]
     nn.clip(rs, 0, h-1, out=rs)
     nn.clip(cs, 0, w-1, out=cs)
@@ -253,7 +272,8 @@ def tile(sample=1, glob=1, window=1024, margin=0.1, astype='float32', progress=p
         def wrap(*p, **key):
             (h, w), ori_img = p[0].shape[:2], p[0]
             samecore = isinstance(ori_img, np.ndarray)
-            img = np.asarray(ori_img, dtype=astype)
+            # img = np.asarray(ori_img, dtype=astype)
+            img = ori_img.astype('float32')
             tps = {'sample', 'window', 'glob', 'margin', 'progress'}
             ftp = fp, tp = {}, {}
             for i in key: ftp[i in tps][i] = key[i]
@@ -276,21 +296,21 @@ def tile(sample=1, glob=1, window=1024, margin=0.1, astype='float32', progress=p
             k = rst.shape[0]/(rcs[0][0].stop - rcs[0][0].start)
             if len(rcs)==1 and ssz!=[h, w]:
                 rst = resize(rst, (int(h*k), int(w*k)))
-            if len(rcs)==1: return rst if samecore else rst.get()
+            if len(rcs)==1: return rst
             def sk(ss, k):
                 sr = slice(int(ss[0].start*k), int(ss[0].stop*k))
                 sc = slice(int(ss[1].start*k), int(ss[1].stop*k))
                 return sr, sc
             outshp = int(img.shape[0]*k), int(img.shape[1]*k)
             outshp = outshp + rst.shape[2:]
-            weights = np.zeros(rst.shape[:2], dtype='uint16')
+            weights = numpy.zeros(rst.shape[:2], dtype='uint16')
             if rst.ndim==3: weights = weights[:,:,None]
             weights += int(mar * k) + 1
             for i in range(int(mar*k), 0, -1):
                 weights[i-1,:] = weights[-i,:] = i
                 weights[:,i-1] = weights[:,-i] = i
-            buf = np.zeros(outshp, dtype=np.float32)
-            count = np.zeros(outshp[:2], dtype='uint16')
+            buf = numpy.zeros(outshp, dtype=np.float32)
+            count = numpy.zeros(outshp[:2], dtype='uint16')
             if rst.ndim==3: count = count[:,:,None]
             buf[sk(rcs[0], k)] = rst * weights
             count[sk(rcs[0], k)] += weights
@@ -299,11 +319,10 @@ def tile(sample=1, glob=1, window=1024, margin=0.1, astype='float32', progress=p
                 rst = f(img[rcs[i]], *p[1:], **fp)
                 buf[sk(rcs[i], k)] += rst * weights
                 count[sk(rcs[i], k)] += weights
-            np.divide(buf, count, out=buf, casting='unsafe')
+            numpy.divide(buf, count, out=buf, casting='unsafe')
             if ssz!=[h, w]: 
                 buf = resize(buf, (int(h*k), int(w*k)))
-            rst = buf.astype(rst.dtype)
-            return rst if samecore else rst.get()
+            return buf.astype(rst.dtype)
         return wrap
     return wrapf
 
